
/* Description

   uses parabolic sar
   
   if sar indicates a trend change, we enter a trade in the direction short/long with the pre-defined stop loss
   if the price hits the last sar point, we close the trade and create an entry below/above the price
   if stop loss is hit, we close the trade and wait for a new trend change
   
*/
//+------------------------------------------------------------------+
//|                                                 Parabolic EA.mq4 |
//|                                 Copyright  2010, Thomas Quester |
//|                                                 www.olfolders.de |
//+------------------------------------------------------------------+
#property copyright "Copyright  2010, Thomas Quester"
#property link      "tquester@gmx.de"

// Beste Werte
// EURUSD 1h CountMA=3, Period = 21

//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
extern double LotSize=0.1;                // Amount to order
double MinMoney=4500;              // Stop everything if we have less then this money
int    StopLoss=50;                // not used?
extern int    TrailingStop=40;            // Trailing stop
extern int    MaxLoss=5;                 // We close trades here
int    m_OldSignal;                // in Cross MA we only trade if the signal changes
int    DrawBars=100;
extern int    TakeProfit=0;              // Set a take profit, may be 0 for no take profit
extern int    Step=1;
extern int    CountMA=10;
extern bool   Trade=true;
extern bool   Alarm=true;
extern bool   Draw=true;
extern bool   UseCrossMA=true;
extern bool   TradeOnStart=true;
extern int    MA_Period=6;
extern int    MA2_Period=36;
extern int    MA_Shift=0;
extern int    MA_Method=0;

      bool   MakeOrders=true;
      bool   Verbose=true;

#define TRADE_INIT            0        // init sar, look if short/long
#define TRADE_WAIT            1
#define TRADE_ENTRY_LONG      2        // warte auf LONG
#define TRADE_ENTRY_SHORT     3        // wait for SHORT
#define TRADE_LONG            4
#define TRADE_SHORT           5
#define TRADE_START           6

#define SAR_SHORT             1
#define SAR_LONG              2


int     m_trade;                          // trade mode
double  m_entry;                          // we wait for a price bigger/smaller than this
double m_startStop;                       // if we are above/below this, we start to watch stop
double m_hiddenStop;                      // our stop level, we do this automatically
                                          // though we order always with stop, the market does not allow it too Close
                                          // but sar sometimes needs closer stops
double sar;                         

int    m_bars;                            // bar count;
bool   m_IsLong;                          // we are long trading, else short 
int    m_Ticket;                          // our order ticket
int    m_TicketCmd;
int    MagicNumber = 22177;               // we caluclate the magic from the symbol and time for example EURUSD5
double m_TotalProfit;                     // Total profit we made
double m_PriceTrade;                      // if we enter a trade, the Price
double m_TrailingStop;                    // Maximum +- TrailingStop pips
double m_Max;                             // Max (or min on short) price
double m_Min;                             // Max (or min on short) price
double m_Open;                            // where we opened the position
string m_LastTrace;
bool   trendchange;
datetime m_TimeTrade;
datetime  lastBlockTime;
double lastBlockPrice;
string m_lastLineName;
double m_LastLongMA;
double m_OrderMaxWin;                     // 

//---- indicator buffers
double ExtMapBuffer[];
double ExtMapBufferLong[];
//----
int ExtCountedBars=0;


int init()
  {
//----
   lastBlockPrice=0;
   Trace("New parabolic EA starting");
   sar = iSAR(NULL,Period(),0.02,0.2,0);
   
   m_trade        = TRADE_WAIT;
   if (TradeOnStart) m_trade=TRADE_START;
  
   m_OldSignal = m_trade;
   m_Ticket = 0;
   m_bars = 0;
   
   MagicNumber = CreateMagic();
   m_Ticket = FindTicket();
   m_PriceTrade = 0;
   
   if (iSAR(NULL,0,0.02,0.2,0) > Ask)
   {
      Trace("new SAR is short");
      m_IsLong = false;
   }
   else
   {
      Trace("new SAR is long");
      m_IsLong = false;
   }
   
   clabel("signal",0,20,"Waiting",White);   
   clabel("label1",0,40,"none",White);   
   clabel("max",0,60,"Max : 0, Profit: 0",White);   
   clabel("win",120,40,"",White);
   clabel("debug",0,80,"Debug",White);   
   clabel("trend",120,20,"Trend",White);   
   ObjectCreate("maxline",OBJ_HLINE,0,0,Ask);
   ObjectCreate("minline",OBJ_HLINE,0,0,Ask);
   ObjectCreate("stopline",OBJ_HLINE,0,0,Ask);
   ObjectCreate("buyline",OBJ_HLINE,0,0,Ask);

   start();
   //----
   return(0);
  }
  
void clabel(string lblname,int x,int y,
string txt,color txtcolor){
ObjectCreate(lblname, OBJ_LABEL,0, 0, 0);
ObjectSet(lblname, OBJPROP_CORNER, 0);
ObjectSetText(lblname,txt,7,"Verdana", txtcolor);
ObjectSet(lblname, OBJPROP_XDISTANCE, x);
ObjectSet(lblname, OBJPROP_YDISTANCE, y);
}

void SetText(string name, string txt)
{
   ObjectSetText(name,txt,7,"Verdana", White);
}
  
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
  {
//----
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
  {
    double win;
   int col;
   int ticket;
   int ticketLong; 
   int    draw_begin;
   int    i;
   int    trade;
   bool isLong;
   
   
   double pip;
   double minstop;
   string str;
   string strShortLong;
   string comment;
   double profit;
   double profitPips;
   double stop;
   string name;   
   double trend,channel50;


   ticket = FindOrder();
   if (ticket != 0) TrailingStop(ticket);
   if (UseCrossMA) HandleStopLoss(m_trade,ticket);

// check if we have a new Bar
   
    if (Bars == m_bars) return (0);
    m_bars = Bars;
    

   // ------------------------------------------------------------------------------------------------
   // -- Calculate MA and draw
   // ------------------------------------------------------------------------------------------------
// Create MA's and draw them    
    trendchange=false; 

    ExtCountedBars = Bars-100;   
    ArrayResize(ExtMapBuffer,Bars);
    ArrayResize(ExtMapBufferLong,Bars);
    if(MA_Period<2) MA_Period=13;
    draw_begin=MA_Period-1;
    DeleteMA();
    switch(MA_Method)
    {
      case 1 : draw_begin=0; break;
      default :
         MA_Method=0;
   }
   switch(MA_Method)
     {
      case 0 : sma();  smaLong(); break;
      case 1 : ema();  emaLong(); break;
      case 2 : smma(); smmaLong(); break;
      case 3 : lwma(); lwmaLong(); break;
     }
   DrawMA();
   trend = MATrend(3,3);
   channel50 = MAChannel(100,100)/Point;
   SetText("trend","Trend="+trend+" Channel100="+channel50);
   m_LastLongMA = ExtMapBufferLong[0];
   sar = iSAR(NULL,0,0.02,0.2,0);   
   
   // ------------------------------------------------------------------------------------------------
   // -- Detect trend change and do acustical Alert
   // -- Draw SAR boxes
   // ------------------------------------------------------------------------------------------------
   
   int cnt;             // count less and count more
   cnt = 0;                 // wir zhlen wie viele Elemente grere und
   //Print("ticket=",ticket, "  cnt=",cnt);

   if (UseCrossMA)
   {
      HandleCrossMA(ticket,trend);
   }
   else
   {  
      HandleSingleMA(ticket,trend); 
   }
   SetText("max","ticket="+ticket+" Max: "+m_Max+" Stop: "+m_TrailingStop+" Min="+m_Min);
   return (0);
}

void HandleSingleMA(int ticket, double trend)
{
   int i;
   int cnt;
   int trade;
   double winShort, winLong;
   winShort = -ExtMapBuffer[CountMA];
   winLong  = ExtMapBuffer[CountMA];
   for (i=CountMA-1;i>=0;i--)
   {
      double d;
      d = ExtMapBuffer[i-1]-ExtMapBuffer[i];
      winShort -= d;
      winLong += d;
   }
   winLong /= Point;
   winShort /= Point;
   SetText("win","Long="+winLong+" Short="+winShort);
   for (i=0;i<CountMA;i++)
   {
      if (ExtMapBuffer[i+1] > ExtMapBuffer[i]) cnt--;
      if (ExtMapBuffer[i+1] < ExtMapBuffer[i]) cnt++;
   }
   
   trade = TRADE_WAIT;
   if (cnt < 0) trade = TRADE_SHORT;
   if (cnt > 0) trade = TRADE_LONG;
   if (ticket == 0) if (MathAbs(cnt) < CountMA/2) trade = TRADE_WAIT;
   Print("cnt = "+cnt+" trade="+trade);
   

   if (m_trade != TRADE_WAIT)
   {   
      if (m_Min != 0)            Rectangle("min",m_Min,Yellow);
      if (m_TrailingStop != 0)   Rectangle("stop",m_TrailingStop,White);
      if (m_Max != 0)            Rectangle("max",m_Max,Magenta) ;
   }
   switch(trade)
   {
       case TRADE_WAIT:
            SetText("signal","Wait");
            if (ticket > 0)
                CloseTrades("Trend Change");
            break;
       case TRADE_LONG:
            if (m_trade == TRADE_SHORT)
                CloseTrades("Trend Change");
            SetText("signal","Long");
            if (ticket == 0) 
                Buy();
          
            break;
       case TRADE_SHORT:
            
            if (m_trade == TRADE_LONG)
                CloseTrades("Trend Change");
            SetText("signal","Short");
            if (ticket == 0) 
               Sell();
           
            break;
    }
    //m_LastLongMA = ExtMapBuffer[0];
}

void HandleCrossMA(int ticket, double trend)
{
      int signal;
      bool buy = true;
      int trade;
       
      double ma_short,ma_long;
      ma_short = ExtMapBuffer[0];
      ma_long =  ExtMapBufferLong[0];
      SetText("debug","m1="+ma_short+" m2="+ma_long);
      if (ma_long > ma_short)
      {
          signal = TRADE_SHORT;
          SetText("signal","SHORT");
          if (trend > 0) 
          {
              buy = false;
              SetText("signal","short but rising");
          }
      }
      else
      {
          signal = TRADE_LONG;
          SetText("signal","LONG");
          if (trend < 0) 
          {
              buy = false;
              SetText("signal","Long but falling");
          }
      }
      
      if (signal == TRADE_SHORT)
      {
          
          if (signal != m_OldSignal)
          {
            if (ticket != 0 && m_TicketCmd == OP_BUY) 
            {
              CloseTrades("Trend Change");
              ticket = 0;
            }
            if (ticket == 0 && buy == true) { m_OldSignal = signal; Sell(); }
          }
          SetText("debug","short");
      }
      if (signal == TRADE_LONG)
      {
          
          if (signal != m_OldSignal)
          {
            if (ticket != 0 && m_TicketCmd == OP_SELL) 
            {
              CloseTrades("Trend Change");
              ticket = 0;
            }

            if (ticket == 0 && buy == true) { m_OldSignal = signal; Buy(); }
          }
          SetText("debug","long");
      }
      
      if (m_OldSignal == TRADE_WAIT)
      {
          m_OldSignal = signal;
          trade = TRADE_WAIT;
      }
      else
      {
          if ((m_OldSignal == TRADE_LONG && signal == TRADE_SHORT)||
              (m_OldSignal == TRADE_SHORT && signal == TRADE_LONG)||
              ((m_OldSignal == TRADE_START && ticket==0)))
          {
              //SetText("debug","Signal Changed");
              trade = signal;
              
          }
      }
}
void HandleStopLoss(int trade, int ticket)
{
   switch(trade)
   {
       case TRADE_LONG:
            if (ticket != 0) 
            {
                if (Ask < m_LastLongMA)
                    CloseTrades("Price hit MA");
          
          /*      
                if (TrailingStop != 0)
                {
                  if (Ask < m_TrailingStop) CloseTrades("Trailing Stop@ "+m_TrailingStop);
                  if (Ask > m_Max) 
                  {
                      m_Max = Ask;
                    
                      m_TrailingStop = Ask - TrailingStop * Point;
                      ObjectSet("maxline",OBJPROP_PRICE1,m_Max);
                      ObjectSet("stopline",OBJPROP_PRICE1,m_TrailingStop);
                  }
                }
                if (MaxLoss != 0)
                {
                  if (Ask < m_Min && m_Min != 0)
                  {
                      CloseTrades("Min@ "+m_Min);
                  }
               }
              */
            }
            break;
       case TRADE_SHORT:
            if (ticket != 0) 
            {
                if (Bid > m_LastLongMA)
                    CloseTrades("Price hit MA");
             /*
                if (TrailingStop != 0)
                {
                  if (Bid > m_TrailingStop) CloseTrades("Trailing Stop@ "+m_TrailingStop);
                  if (Bid < m_Max) 
                  {
                      m_Max = Bid;
                      m_TrailingStop = Bid + TrailingStop * Point;
                      ObjectSet("maxline",OBJPROP_PRICE1,m_Max);
                      ObjectSet("stopline",OBJPROP_PRICE1,m_TrailingStop);
                  }
                }
                if (MaxLoss > 0)
                {
                  if (Bid > m_Min && m_Min != 0)
                  {
                      CloseTrades("Min@ "+m_Min);
                  }
                }
               */ 
            }  
            break;
   }
}
void Buy()
{
  double stop,profit;
  string comment;
  if (!Trade) return;
  
  stop = Ask-Point*StopLoss;
  if (sar < stop)
      stop = sar;
  if (TakeProfit != 0)
       profit = Ask+TakeProfit*Point;
  else  
      profit=0;
   Trace("Entering long stop="+stop+ " Take Profit="+profit);
   comment = WindowExpertName()+" BUY "+Period();
   m_Ticket = OrderSend(Symbol(), OP_BUY, LotSize, Ask,0, stop, profit, comment,MagicNumber,0,Blue);
   m_OrderMaxWin = Ask;
   m_Max    = 0;                        // maximum price, the trailing stop is m_Max-TrailingStop
   m_Min    = Ask-Point*MaxLoss;
   m_TrailingStop = m_Max-TrailingStop;   // trailing stop
   
   m_PriceTrade = Ask;
   if (Verbose) Trace("Ticket = "+m_Ticket+ " Error="+GetLastError());
   if (m_Ticket > 0)
       m_trade = TRADE_LONG;
}

void Sell()
{
   double stop,profit;
   string comment;
   if (!Trade) return;   
   
   stop = Bid+StopLoss*Point;
   if (sar > stop)
       stop = sar;
   if (TakeProfit != 0)
       profit = Bid-TakeProfit*Point;
   else  
       profit=0;
   m_Max = 99999;
   if (Verbose) Trace("Entering short stop="+stop+ " Take Profit="+profit); 
   comment = WindowExpertName()+" SELL "+Period();  
   m_Ticket = OrderSend(Symbol(), OP_SELL, LotSize, Bid,0, stop, profit, comment,MagicNumber,0,Red);
   m_Max    =  Bid;
   m_Min    = Bid+Point*MaxLoss;
   m_TrailingStop = Bid+TrailingStop*Point;
   m_OrderMaxWin = Bid;
   m_PriceTrade = Bid;
   Trace("Ticket = "+m_Ticket+ " Error="+GetLastError());
   if (m_Ticket > 0)
       m_trade = TRADE_SHORT;
}

void TrailingStop(int ticket)
{
   int cmd;
   double stop,newStop;
   if (TrailingStop == 0) return;
   if (OrderSelect(ticket,SELECT_BY_TICKET,MODE_TRADES)==true)
   {
       
       stop = OrderStopLoss();
       cmd = OrderType();
       
       if (cmd == OP_BUY)
       {
           if (Ask > m_OrderMaxWin)
           {
              Print ("Trailing found order Ask=",Ask," Bid=",Bid," Max=",m_OrderMaxWin);
              SetText("max",Ask);
              m_OrderMaxWin = Ask;
              newStop = m_OrderMaxWin - Point*TrailingStop;
              if (newStop > stop)
                  OrderModify(ticket,OrderOpenPrice(),newStop,OrderTakeProfit(),0,Blue);
           }
       } 
       if (cmd == OP_SELL)
       {
           if (Bid < m_OrderMaxWin)
           {
              Print ("Trailing found order Ask=",Ask," Bid=",Bid," Max=",m_OrderMaxWin);
              SetText("max",Bid);
              m_OrderMaxWin = Bid;
              newStop = m_OrderMaxWin + Point*TrailingStop;
              if (newStop < stop)
                  OrderModify(ticket,OrderOpenPrice(),newStop,OrderTakeProfit(),0,Blue);
           }
       }
   }  
   else  
     Print("Trailing error open order "+ticket);  
       
}

//+------------------------------------------------------------------+

double MATrend(int start, int count)
{
   double r;
   double v;
   int    i;
   r = 0;
   v = ExtMapBuffer[start-count];
   for (i=start-count+1;i<=start;i++)
   {
       r = r + v-ExtMapBuffer[i];
   }
   return (r);
}

double MAChannel(int start, int count)
{
   double min,max,r;
   double v;
   int    i;
   r = 0;
   min = 99999;
   max = -9999;
   v = ExtMapBuffer[start-count];
   for (i=start-count+1;i<=start;i++)
   {
       v = ExtMapBuffer[i];
       if (v < min) min = v;
       if (v > max) max = v;
   }
   return (max-min);
}

int CreateMagic()
{
   string s = "MA"+Symbol()+Period();
   string s1;
   int c;
   int i;
   int sum =0;
   for (i=0;i<StringLen(s);i++)
   {
      
      c = StringGetChar(s,i);
      sum = sum + c;
   }
   
   Trace("Magic="+sum+" s="+s);
   return (sum);
 }
      
      
int FindTicket()
{
   for (int i=OrdersTotal()-1; i>=0; i--)
   {                                               
      if (OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==true)
      {
         if (OrderMagicNumber() == MagicNumber) 
         {
             if (Verbose) Trace("Found Order "+OrderTicket());
             return (OrderTicket());
         }
      }   
   }
   return (0);
}      


int CloseAll()
  {
   bool   result;
   double price;
   int    cmd,error;
//----
      for (int i=OrdersTotal()-1; i>=0; i--)
      {                                               
         if (OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==true)
         {

               Trace("Closing order");
               cmd=OrderType();
               if(cmd==OP_BUY) price=Bid;
               else            price=Ask;
               result=OrderClose(OrderTicket(),OrderLots(),price,300,CLR_NONE);
               if(result!=TRUE) { error=GetLastError(); Trace("LastError = "+error); }
            
         }   
      }

   
   return(0);
  }


void CloseTrades(string comment)
{
   Print("************** Close Trades : ",comment);
   SetText("label2",comment);
   CloseOrders(CreateMagic());
}

int CloseOrders(int magic)
  {
   bool   result;
   double price;
   int    cmd,error;
   m_Min = 0;
   m_Max = 0;
   m_TrailingStop = 0;
   m_trade = TRADE_WAIT;
//----
      for (int i=OrdersTotal()-1; i>=0; i--)
      {                                               
         if (OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==true)
         {
           if (OrderMagicNumber() == magic)
           {

               Trace("Closing order");
               cmd=OrderType();
               if(cmd==OP_BUY) price=Bid;
               else            price=Ask;
               result=OrderClose(OrderTicket(),OrderLots(),price,300,CLR_NONE);
               if(result!=TRUE) { error=GetLastError(); Trace("LastError = "+error); }
           }
            
         }   
      }

   
   return(0);
  }



int FindOrder()
{
   int i;
   int cmd;
   int ticket;
   ticket=0;
   string str;
   for (i=OrdersTotal()-1; i>=0; i--)
   {                                               
         if (OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==true)
         {
            if (OrderMagicNumber() == MagicNumber) 
            {
               cmd = OrderType();
               m_TicketCmd = cmd;
               if (cmd == OP_BUY)  { m_IsLong = true;    m_trade = TRADE_LONG;  }
               if (cmd == OP_SELL) { m_IsLong = false;   m_trade = TRADE_SHORT; }
               ticket = OrderTicket();
               m_PriceTrade = OrderOpenPrice();
               if (m_Max == 0)
               {
                   m_TrailingStop = sar;
                   if (cmd == OP_BUY)
                       m_Max = m_TrailingStop + TrailingStop*Point;
                   else
                       m_Max = m_TrailingStop - TrailingStop*Point;
               }
//               Trace("Order found ",ticket," ",str," ",OrderComment());
            }
         }   
   }
   return (ticket);
}

void SetStopLoss(int pips)
{
    double stop;
    double min;
    min = MarketInfo( Symbol(), MODE_STOPLEVEL); 
    OrderSelect(m_Ticket,SELECT_BY_TICKET );
    if (OrderType() == OP_BUY) stop = Ask-min*Point;
    if (OrderType() == OP_SELL) stop = Bid+min*Point;
    OrderModify(m_Ticket,OrderOpenPrice(),stop,OrderTakeProfit(),0,Blue);
}

void Trace(string s)
{
   if (s != m_LastTrace) Print(s);
   m_LastTrace=s;
}

void SetStop(double stop)
{
    double d;
    string name;
    name = "stop"+Time[0];
    d = stop;
    /*
    if (m_IsLong)
    {
       if (stop > m_hiddenStop || m_hiddenStop == 0) d = stop;
    }
    else
    {   
       if (stop < m_hiddenStop || m_hiddenStop == 0) d = stop;
    }
    */
    if (d != 0)
    {
         ObjectCreate(name, OBJ_RECTANGLE, 0, Time[0],d,Time[2],d+Point/3);
         ObjectSet(name,OBJPROP_COLOR,Yellow);
    
         m_hiddenStop = d;
         if (m_Ticket >= 0)
           OrderModify(m_Ticket,OrderOpenPrice(),m_hiddenStop,OrderTakeProfit(),0,Blue);
    }
}      




//+------------------------------------------------------------------+
//| Simple Moving Average                                            |
//+------------------------------------------------------------------+
void sma()
  {
   double sum=0;
   int    i,pos=Bars-ExtCountedBars-1;
//---- initial accumulation
   if(pos<MA_Period) pos=MA_Period;
   for(i=1;i<MA_Period;i++,pos--)
      sum+=Close[pos];
//---- main calculation loop
   while(pos>=0)
     {
      sum+=Close[pos];
      ExtMapBuffer[pos]=sum/MA_Period;
	   sum-=Close[pos+MA_Period-1];
 	   pos--;
     }
//---- zero initial bars
   if(ExtCountedBars<1)
      for(i=1;i<MA_Period;i++) ExtMapBuffer[Bars-i]=0;
  }
  
void smaLong()
  {
   double sum=0;
   int    i,pos=Bars-ExtCountedBars-1;
//---- initial accumulation
   if(pos<MA2_Period) pos=MA2_Period;
   for(i=1;i<MA2_Period;i++,pos--)
      sum+=Close[pos];
//---- main calculation loop
   while(pos>=0)
     {
      sum+=Close[pos];
      ExtMapBufferLong[pos]=sum/MA2_Period;
	   sum-=Close[pos+MA2_Period-1];
 	   pos--;
     }
//---- zero initial bars
   if(ExtCountedBars<1)
      for(i=1;i<MA2_Period;i++) ExtMapBufferLong[Bars-i]=0;
  }  
//+------------------------------------------------------------------+
//| Exponential Moving Average                                       |
//+------------------------------------------------------------------+
void ema()
  {
   double pr=2.0/(MA_Period+1);
   int    pos=Bars-2;
   if(ExtCountedBars>2) pos=Bars-ExtCountedBars-1;
//---- main calculation loop
   while(pos>=0)
     {
      if(pos==Bars-2) ExtMapBuffer[pos+1]=Close[pos+1];
      ExtMapBuffer[pos]=Close[pos]*pr+ExtMapBuffer[pos+1]*(1-pr);
 	   pos--;
     }
  }
  
  
void emaLong()
  {
   double pr=2.0/(MA2_Period+1);
   int    pos=Bars-2;
   if(ExtCountedBars>2) pos=Bars-ExtCountedBars-1;
//---- main calculation loop
   while(pos>=0)
     {
      if(pos==Bars-2) ExtMapBufferLong[pos+1]=Close[pos+1];
      ExtMapBuffer[pos]=Close[pos]*pr+ExtMapBufferLong[pos+1]*(1-pr);
 	   pos--;
     }
  }  
  
//+------------------------------------------------------------------+
//| Smoothed Moving Average                                          |
//+------------------------------------------------------------------+
void smma()
  {
   double sum=0;
   int    i,k,pos=Bars-ExtCountedBars+1;
//---- main calculation loop
   pos=Bars-MA_Period;
   if(pos>Bars-ExtCountedBars) pos=Bars-ExtCountedBars;
   while(pos>=0)
     {
      if(pos==Bars-MA_Period)
        {
         //---- initial accumulation
         for(i=0,k=pos;i<MA_Period;i++,k++)
           {
            sum+=Close[k];
            //---- zero initial bars
            ExtMapBuffer[k]=0;
           }
        }
      else sum=ExtMapBuffer[pos+1]*(MA_Period-1)+Close[pos];
      ExtMapBuffer[pos]=sum/MA_Period;
 	   pos--;
     }
  }
  
void smmaLong()
  {
   double sum=0;
   int    i,k,pos=Bars-ExtCountedBars+1;
//---- main calculation loop
   pos=Bars-MA2_Period;
   if(pos>Bars-ExtCountedBars) pos=Bars-ExtCountedBars;
   while(pos>=0)
     {
      if(pos==Bars-MA2_Period)
        {
         //---- initial accumulation
         for(i=0,k=pos;i<MA2_Period;i++,k++)
           {
            sum+=Close[k];
            //---- zero initial bars
            ExtMapBufferLong[k]=0;
           }
        }
      else sum=ExtMapBufferLong[pos+1]*(MA2_Period-1)+Close[pos];
      ExtMapBufferLong[pos]=sum/MA2_Period;
 	   pos--;
     }
  }  
//+------------------------------------------------------------------+
//| Linear Weighted Moving Average                                   |
//+------------------------------------------------------------------+
void lwma()
  {
   double sum=0.0,lsum=0.0;
   double price;
   int    i,weight=0,pos=Bars-ExtCountedBars-1;
//---- initial accumulation
   if(pos<MA_Period) pos=MA_Period;
   for(i=1;i<=MA_Period;i++,pos--)
     {
      price=Close[pos];
      sum+=price*i;
      lsum+=price;
      weight+=i;
     }
//---- main calculation loop
   pos++;
   i=pos+MA_Period;
   while(pos>=0)
     {
      ExtMapBuffer[pos]=sum/weight;
      if(pos==0) break;
      pos--;
      i--;
      price=Close[pos];
      sum=sum-lsum+price*MA_Period;
      lsum-=Close[i];
      lsum+=price;
     }
//---- zero initial bars
   if(ExtCountedBars<1)
      for(i=1;i<MA_Period;i++) ExtMapBuffer[Bars-i]=0;
  }
  
void lwmaLong()
  {
   double sum=0.0,lsum=0.0;
   double price;
   int    i,weight=0,pos=Bars-ExtCountedBars-1;
//---- initial accumulation
   if(pos<MA2_Period) pos=MA2_Period;
   for(i=1;i<=MA2_Period;i++,pos--)
     {
      price=Close[pos];
      sum+=price*i;
      lsum+=price;
      weight+=i;
     }
//---- main calculation loop
   pos++;
   i=pos+MA2_Period;
   while(pos>=0)
     {
      ExtMapBufferLong[pos]=sum/weight;
      if(pos==0) break;
      pos--;
      i--;
      price=Close[pos];
      sum=sum-lsum+price*MA2_Period;
      lsum-=Close[i];
      lsum+=price;
     }
//---- zero initial bars
   if(ExtCountedBars<1)
      for(i=1;i<MA2_Period;i++) ExtMapBufferLong[Bars-i]=0;
  }  
//+------------------------------------------------------------------+


void DeleteMA()
{
  int i;
  string name;
  if (!Draw) return;
  for (i=0;i<DrawBars;i++)
  {
     name = "MA"+i;
     ObjectDelete(name);
  }
//  if (UseCrossMA)
  {
   for (i=0;i<DrawBars;i++)
   {
      name = "MAL"+i;
      ObjectDelete(name);
   }
  }
}

void Rectangle(string name, double price, int col)
{
    string name1;
    name1 = name + Time[0];
    ObjectCreate(name1, OBJ_RECTANGLE, 0, Time[1],price,Time[2],price+Point);
    ObjectSet(name1,OBJPROP_COLOR,col);
}

void DrawMA()
{
  int i;
  string name;
  if (!Draw) return;
  int start = Bars;
  if (start > DrawBars)
      start = DrawBars;
  for (i=start;i>=0;i--)
  {
     name = "MA"+i;
     ObjectCreate(name, OBJ_RECTANGLE, 0, Time[i],ExtMapBuffer[i],Time[i+1],ExtMapBuffer[i]+Point);
//     if (UseCrossMA)
     {
         name = "MAL"+i;
         ObjectCreate(name, OBJ_RECTANGLE, 0, Time[i],ExtMapBufferLong[i],Time[i+1],ExtMapBufferLong[i]+Point);
         ObjectSet(name,OBJPROP_COLOR,Red);
     }
  }
  /*
  if (UseCrossMA)
  {
   for (i=start;i>=0;i--)
   {
      name = "MAL"+i;
      ObjectCreate(name, OBJ_RECTANGLE, 0, Time[i],ExtMapBufferLong[i],Time[i+1],ExtMapBufferLong[i]+Point);
      ObjectSet(name,OBJPROP_COLOR,Red);
   }
  }
  */
}   